#!/usr/bin/python
"""Run an executable on a platform

Usage:
    platformrun [-v | -vv] [options] PLATFORM EXECUTABLE
    platformrun -h

Options:
    -h --help           Show this usage message
    -c --config CONF    Specify the measurement configuration to load
                            [default: ~/.measurementrc]
    -t --tools CONF     Config file for the tools needed to run on a platform
                            [default: ~/.platformrunrc]
    -r --repeat N       Repeat the test N times [default: 1]
    -v --verbose        Be verbose
    --no-measure        Don't measure anything
    --csv               Print as csv
    -s                  Split energy measurements when board using multiple
                        measurement points

    PLATFORM        Specify the platform on which to run.
                    Available platforms are:
                        stm32f0discovery
                        stm32l0discovery
                        stm32vldiscovery
                        stm32f4discovery
                        atmega328p
                        xmegaa3buxplained
                        msp-exp430f5529
                        msp-exp430fr5739
                        pic32mx250f128b
                        sam4lxplained

"""
from docopt import docopt

import subprocess, os, os.path
import threading
from collections import namedtuple
import tempfile
import json

import pyenergy
from time import sleep
import logging
import usb

import pexpect, sys, copy

logger = logging.getLogger(__name__)
warning = logger.warning
info = logger.info
debug = logger.debug


tool_config = None
measurement_config = None

#######################################################################

class CommandError(RuntimeError):
    pass

class LogWriter(object):
    def __init__(self, logger, level):
        self.logger = logger
        self.level = level
        self.current_msg = ""
    def write(self, msg):
        self.current_msg += msg

        if '\n' in self.current_msg:
            s = self.current_msg.split('\n')
            to_print = s[:-1]
            self.current_msg = s[-1]
            for s in to_print:
                self.logger.log(self.level, s)
    def flush(self):
        pass
    def close(self):
        pass

def gdb_launch(gdbname, port, fname, run_timeout=30, post_commands=[], pre_commands=[]):
    info("Starting+connecting to gdb and loading file")

    gdblogger = logger.getChild(os.path.split(gdbname)[-1])
    gdb = pexpect.spawn(gdbname, logfile=LogWriter(gdblogger, logging.DEBUG))

    gdb.expect(r".*\(gdb\) ")

    gdb.sendline("set confirm off")
    gdb.expect(r".*\(gdb\) ")

    gdb.sendline("target remote :{}".format(port))
    gdb.expect(r'Remote debugging using.*\n')
    gdb.expect(r'.*\n')
    gdb.expect(r".*\(gdb\) ")

    for cmd in pre_commands:
        gdb.sendline(cmd)
        gdb.expect(r'.*\(gdb\) ')

    info("load file {}".format(fname))
    gdb.sendline("file {}".format(fname))
    gdb.expect(r'Reading symbols.*\n')
    gdb.expect(r".*\(gdb\) ")

    gdb.sendline("load")
    while True:
        a = gdb.expect([r'Loading section.*\n',r'Start address.*\n', r"Error finishing flash operation.*\n"])
        if a == 1:
            break
        if a == 2:
            gdb.expect(r".*\(gdb\) ")
            gdb.sendline("load")

    gdb.expect(r'Transfer rate.*\n')
    gdb.expect(r'\(gdb\) ')

    gdb.sendline("delete breakpoints")
    gdb.expect(r".*\(gdb\) ")

    gdb.sendline("break exit")
    gdb.expect(r'.*Breakpoint .*\n')
    gdb.expect(r".*\(gdb\) ")

    gdb.sendline("break _exit")
    gdb.expect(r'.*Breakpoint .*\n')
    gdb.expect(r".*\(gdb\) ")

    info("Sending continue to gdb")
    gdb.sendline("continue")
    gdb.expect(r'Continuing.*\n')
    gdb.expect(r'.*Breakpoint.*exit.*\n', timeout=run_timeout)

    info("Breakpoint hit")

    for cmd in post_commands:
        gdb.sendline(cmd)
        gdb.expect(r'.*\(gdb\) ')

    info("Quitting GDB")
    gdb.sendline("quit")

bg_procs = []
bg_proc_map = {}

def background_proc(cmd, env=None):

    def run_proc(ev):
        info("Starting background proc: \"{}\"".format(cmd))

        proclogger = logger.getChild(os.path.split(cmd.split(' ')[0])[-1])
        proc = pexpect.spawn(cmd, logfile=LogWriter(proclogger, logging.DEBUG), env=env)

        while not ev.isSet():
            proc.expect([r'.*\n', pexpect.EOF, pexpect.TIMEOUT], timeout=1)
            sleep(0.1)

        info("Killing background proc: \"{}\"".format(cmd))
        proc.kill(15)
        sleep(0.2)
        if proc.isalive():
            warning("Killing (sig 9) background proc: \"{}\"".format(cmd))
            proc.kill(9)

    ev = threading.Event()
    t = threading.Thread(target=run_proc, args=(ev,))
    t.start()

    bg_procs.append(ev)
    bg_proc_map[ev] = t

    return ev

def kill_background_proc(p):
    p.set()
    info("Waiting for background process to exit " + str(p))
    bg_proc_map[p].join()
    sleep(0.1)
    bg_procs.remove(p)

def killBgOnCtrlC(f):
    def wrap(platform, *args, **kwargs):
        try:
            return f(platform, *args, **kwargs)
        except:
            info("Exception occured, killing background procs")
            for p in copy.copy(bg_procs):
                kill_background_proc(p)
            raise
    return wrap


def foreground_proc(cmd, expected_returncode=0):
    info("Starting foreground proc: \"{}\"".format(cmd))

    proclogger = logger.getChild(cmd.split(' ')[0])

    proc = pexpect.spawn(cmd, logfile=LogWriter(proclogger, logging.DEBUG))

    proc.expect(pexpect.EOF)

    # Even though we have received an EOF, the process may not be
    # fully dead. Therefore, we wait. An exception is raised if the
    # process is already dead
    try:
        proc.wait()
    except pexpect.ExceptionPexpect:
        pass

    if expected_returncode is not None and proc.exitstatus != expected_returncode:
        raise CommandError("Command \"{}\" returned {}".format(cmd, proc.exitstatus))
    info("Foreground proc complete")


def setupMeasurement(platform, doMeasure=True):
    if not doMeasure:
        return None

    # First check that all tools for platform are available
    if platform not in tool_config['enabled']:
        raise RuntimeError("Platform {0} does not have all the tools configured. Please rerun configure with --enable-{0}".format(platform))

    em = pyenergy.EnergyMonitor(measurement_config[platform]['energy-monitor'])

    mp_cfg = measurement_config[platform]['measurement-point']
    if type(mp_cfg) is int:
        mpoints = [mp_cfg]
        resistors = [measurement_config[platform]['resistor']]
    elif type(mp_cfg) is list:
        mpoints = mp_cfg
        resistors = measurement_config[platform]['resistor']
    # mp = int(measurement_config[platform]['measurement-point'])

    em.connect()
    for mp, resistor in zip(mpoints, resistors):
        em.enableMeasurementPoint(mp)
        em.clearNumberOfRuns(mp)
        em.measurement_params[mp]['resistor'] = float(resistor)
        em.setTrigger(measurement_config[platform]['trigger-pin'], mp)

    return em

def finishMeasurement(platform, em, doMeasure=True, timeout=30):
    if not doMeasure:
        return None

    mp_cfg = measurement_config[platform]['measurement-point']
    if type(mp_cfg) is int:
        mpoints = [mp_cfg]
        resistors = [measurement_config[platform]['resistor']]
    elif type(mp_cfg) is list:
        mpoints = mp_cfg
        resistors = measurement_config[platform]['resistor']

    info("Waiting for measurement to complete")
    t = 0
    for mp in mpoints:
        while not em.measurementCompleted(mp):
            sleep(0.1)
            t += 1
            if t > timeout * 10:
                raise CommandError("Measurement timeout")
    info("Measurement complete")


    if type(mp_cfg) is list:
        measurements = map(em.getMeasurement, mpoints)
        m = pyenergy.MeasurementSet(measurements)
    else:
        m = em.getMeasurement(mp)

    em.disconnect()
    return m

# Display units nicer
def prettyPrint(v):
    units = [' ', 'm', 'u', 'n', 'p']

    for unit in units:
        if v > 1.0:
            return "{: >8.3f} {}".format(v, unit)
        v *= 1000.
    return "{}".format(v)

#######################################################################

def loadConfiguration(fname="~/.measurementrc"):
    global measurement_config

    fname = os.path.expanduser(fname)
    measurement_config = json.load(open(fname))

def loadToolConfiguration(fname="~/.platformrunrc"):
    global tool_config

    fname = os.path.expanduser(fname)
    tool_config = json.load(open(fname))

##

def findUSBLocation(devid):
    info("Discovering USB bus address of {}".format(devid))

    for bus in usb.busses():
        for dev in bus.devices:
            try:
                print
                if devid.lower() == "{:04x}:{:04x}".format(dev.idVendor, dev.idProduct):
                    addr = "{:03}:{:03}".format(dev.dev.bus, dev.dev.address)
                    info("Found {}".format(addr))
                    return addr
            except usb.core.USBError:
                pass
    warning("Could not find device location for {}. The wrong board may be selected".format(addr))
    return ""


#######################################################################

@killBgOnCtrlC
def stm32f0discovery(fname, doMeasure=True):
    em = setupMeasurement("stm32f0discovery", doMeasure)

    stproc = background_proc(tool_config['tools']['stutil'] + " -s 2 -p 2001 -c 0x0bb11477 -v0")
    gdb_launch(tool_config['tools']['arm_gdb'], 2001, fname)
    kill_background_proc(stproc)

    return finishMeasurement("stm32f0discovery", em, doMeasure)


@killBgOnCtrlC
def stm32vldiscovery(fname, doMeasure=True):
    em = setupMeasurement("stm32vldiscovery", doMeasure)

    stproc = background_proc(tool_config['tools']['stutil'] + " -s 1 -p 2002 -c 0x1ba01477 -v0")
    gdb_launch(tool_config['tools']['arm_gdb'], 2002, fname)
    kill_background_proc(stproc)

    return finishMeasurement("stm32vldiscovery", em, doMeasure)

@killBgOnCtrlC
def stm32f4discovery(fname, doMeasure=True):
    em = setupMeasurement("stm32f4discovery", doMeasure)

    stproc = background_proc(tool_config['tools']['stutil'] + " -s 2 -p 2003 -c 0x2ba01477 -v0")
    gdb_launch(tool_config['tools']['arm_gdb'], 2003, fname)
    kill_background_proc(stproc)

    return finishMeasurement("stm32f4discovery", em, doMeasure)

@killBgOnCtrlC
def stm32l0discovery(fname, doMeasure=True):
    em = setupMeasurement("stm32l0discovery", doMeasure)

    stproc = background_proc(tool_config['tools']['openocd'] + ' -f board/stm32l0discovery.cfg --command "gdb_port 2004"')
    gdb_launch(tool_config['tools']['arm_gdb'], 2004, fname, post_commands=["monitor shutdown"], pre_commands=["monitor reset init"])
    kill_background_proc(stproc)

    return finishMeasurement("stm32l0discovery", em, doMeasure)

@killBgOnCtrlC
def beaglebone(fname, doMeasure=True):
    em = setupMeasurement("beaglebone", doMeasure)

    openocdproc = background_proc(tool_config['tools']['openocd'] + " -f board/ti_beaglebone.cfg")
    gdb_launch(tool_config['tools']['arm_gdb'], 3333, fname, post_commands=["monitor shutdown"], pre_commands=["monitor reset halt"])
    kill_background_proc(openocdproc)

    return finishMeasurement("beaglebone", em, doMeasure)


def atmega328p(fname, doMeasure=True):
    # Create temporary file and convert to hex file
    tf = tempfile.NamedTemporaryFile(delete=False)
    tf.close()
    foreground_proc("{} -O ihex {} {}".format(tool_config['tools']['avr_objcopy'], fname, tf.name))

    # Flash the hex file to the AVR chip
    if logger.getEffectiveLevel() == logging.DEBUG:
        silence = ""
    elif logger.getEffectiveLevel() == logging.INFO:
        silence = "-q"
    else:
        silence = "-q -q"

    ser_id = measurement_config['atmega328p']['serial-dev-id']
    ser_path = "/dev/serial/by-id/{}".format(ser_id)

    if not os.path.exists(ser_path):
        raise RuntimeError("Cannot find the serial device: "+ser_path)

    cmdline = "readlink -m " + ser_path
    location = pexpect.run(cmdline).strip()
    info("AVR programmer @ {}".format(location))

    # info("Perform erase of the chip")
    # cmdline = "{} -F -c arduino -p atmega328p -e -P {} -b 115200".format(tool_config['tools']['avrdude'], location)
    # foreground_proc(cmdline)

    em = setupMeasurement("atmega328p", doMeasure)

    cmdline = "{} -F -V -c arduino -p atmega328p -P {} -b 115200 -U flash:w:{}".format(tool_config['tools']['avrdude'], location, tf.name)
    foreground_proc(cmdline)

    try:
        os.unlink(tf.name)
    except OSError:
        pass
    return finishMeasurement("atmega328p", em, doMeasure)

def xmegaa3buxplained(fname, doMeasure=True):
    em = setupMeasurement("xmegaa3buxplained", doMeasure)

    # Create temporary file and convert to hex file
    tf = tempfile.NamedTemporaryFile(delete=False)
    tf.close()
    foreground_proc("{} -O ihex {} {}".format(tool_config['tools']['avr_objcopy'], fname, tf.name))

    # Flash the hex file to the AVR chip
    if logger.getEffectiveLevel() == logging.DEBUG:
        silence = ""
    elif logger.getEffectiveLevel() == logging.INFO:
        silence = "-q"
    else:
        silence = "-q -q"

    cmdline = "{} -F -V -c jtag3 -p x256a3bu -e -U flash:w:{}".format(tool_config['tools']['avrdude'], tf.name)
    foreground_proc(cmdline)

    try:
        os.unlink(tf.name)
    except OSError:
        pass
    return finishMeasurement("xmegaa3buxplained", em, doMeasure)


def mspexp430f5529(fname, doMeasure=True):
    em = setupMeasurement("msp-exp430f5529", doMeasure)

    foreground_proc("{} tilib -q \"prog {}\" &".format(tool_config['tools']['mspdebug'], fname))

    return finishMeasurement("msp-exp430f5529", em, doMeasure)


def mspexp430fr5739(fname, doMeasure=True):
    em = setupMeasurement("msp-exp430fr5739", doMeasure)

    foreground_proc("{} rf2500 \"prog {}\" &".format(tool_config['tools']['mspdebug'], fname), expected_returncode=None)

    return finishMeasurement("msp-exp430fr5739", em, doMeasure)


def pic32mx250f128b(fname, doMeasure=True):
    tf = tempfile.NamedTemporaryFile(delete=False)
    tf.write(":00000001FF")
    tf.close()

    foreground_proc("{} -S -p {}".format(tool_config['tools']['pic32prog'], tf.name))
    sleep(0.2)

    if len(fname) > 4 and fname[-4:] == ".hex":
        tfname = fname
    else:
        # Create temporary file and convert to hex fil, doMeasuree
        foreground_proc("{} -O ihex {} {}".format(tool_config['tools']['pic32_objcopy'], fname, tf.name))
        tfname = tf.name

    # Program the PIC and leave power on to run test
    em = setupMeasurement("pic32mx250f128b")
    foreground_proc("{} -S -p {}".format(tool_config['tools']['pic32prog'], tfname))

    os.unlink(tf.name)
    return finishMeasurement("pic32mx250f128b", em, doMeasure)


@killBgOnCtrlC
def sam4lxplained(fname, doMeasure=True):
    em = setupMeasurement("sam4lxplained", doMeasure)

    openocdproc = background_proc(tool_config['tools']['openocd'] + " -f board/atmel_sam4l8_xplained_pro.cfg")
    gdb_launch(tool_config['tools']['arm_gdb'], 3333, fname, post_commands=["monitor shutdown"])
    kill_background_proc(openocdproc)

    return finishMeasurement("sam4lxplained", em, doMeasure)


def run(platformname, execname, measurement=True):

    if not os.path.isfile(execname):
        raise IOError("File \"{}\" does not exist".format(execname))

    if platformname == "stm32f0discovery":
        m = stm32f0discovery(execname, measurement)
    elif platformname == "stm32vldiscovery":
        m = stm32vldiscovery(execname, measurement)
    elif platformname == "stm32f4discovery":
        m = stm32f4discovery(execname, measurement)
    elif platformname == "stm32l0discovery":
        m = stm32l0discovery(execname, measurement)
    elif platformname == "beaglebone":
        m = beaglebone(execname, measurement)
    elif platformname == "atmega328p":
        m = atmega328p(execname, measurement)
    elif platformname == "xmegaa3buxplained":
        m = xmegaa3buxplained(execname, measurement)
    elif platformname == "pic32mx250f128b":
        m = pic32mx250f128b(execname, measurement)
    elif platformname == "msp-exp430f5529":
        m = mspexp430f5529(execname, measurement)
    elif platformname == "msp-exp430fr5739":
        m = mspexp430fr5739(execname, measurement)
    elif platformname == "sam4lxplained":
        m = sam4lxplained(execname, measurement)
    else:
        raise RuntimeError("Unknown platform " + platformname)
    return m


def run_multiple(platformname, execname, repeats, measurement=True):
    measurements = []

    for i in range(repeats):
        m = run(platformname, execname, measurement)
        measurements.append(m)

    return sum(measurements) / len(measurements)

def main():
    arguments = docopt(__doc__)

    logging.basicConfig()

    if arguments['--verbose'] == 1:
        logging.getLogger('').setLevel(logging.INFO)
    elif arguments['--verbose']== 2:
        logging.getLogger('').setLevel(logging.DEBUG)

    loadConfiguration(arguments['--config'])
    loadToolConfiguration(arguments['--tools'])

    try:
        # Execute multiple repeats, if requested
        ms = []
        for i in range(int(arguments['--repeat'])):
            m = run(arguments['PLATFORM'],
                arguments['EXECUTABLE'],
                arguments['--no-measure'] is False)
            ms.append(m)

        m = sum(ms) / len(ms)
    except (IOError, RuntimeError) as e:
        print "Error:",e
        quit(1)

    if arguments['--no-measure'] is False:
        if arguments['--csv']:
            print "{m.energy}, {m.time}, {m.avg_power}, {m.avg_current}, {m.avg_voltage}".format(m=m)
        else:
            if arguments['-s'] and hasattr(m,"measurements"):

                s = "       "
                for i in range(len(m.measurements)):
                    s += "           {}".format(i+1)
                print s

                names = ["Energy", "Time", "Power", "Average current", "Average voltage"]
                keys = ["energy", "time", "avg_power", "avg_current", "avg_voltage"]
                units = ["J", "s", "W", "A", "V"]

                for name,key,unit in zip(names, keys, units):
                    print "{}:{}".format(name, " " *(15-len(name))),
                    for meas in m.measurements:
                        print "{}{}".format(prettyPrint(meas.__dict__[key]), unit),
                    print

            else:
                print "Energy:          {}J".format(prettyPrint(m.energy))
                print "Time:            {}s".format(prettyPrint(m.time))
                print "Power:           {}W".format(prettyPrint(m.avg_power))
                print "Average current: {}A".format(prettyPrint(m.avg_current))
                print "Average voltage: {}V".format(prettyPrint(m.avg_voltage))

if __name__ == "__main__":
    main()



#!/usr/bin/python

"""Energy tools

Usage:
    energytool (-m MPOINT)... [options] read SERIAL PIN
    energytool (-m MPOINT)... [options] continuous SERIAL
    energytool (-m MPOINT)... [options] debug SERIAL
    energytool list
    energytool setup
    energytool changeserial SERIAL NEWSERIAL
    energytool interactive SERIAL

Commands:
    read            This sets up a trigger on the specified PIN and waits for
                    an energy measurement.

    continuous      Continuously read measurements from the specified energy
                    monitor.

    debug           Output some debug data about the instantaneous voltages
                    seen on the ADCs, along with current and voltage.

    list            Show the serial numbers and API version of each connected
                    energy monitor.

    setup           Prompt about the energy measurement configuration and
                    create a .measurementrc file.

    changeserial    Connect to the device specified by SERIAL, and change the
                    serial to NEWSERIAL

    interactive     Start up the interactive graph view, for energy monitor
                    SERIAL. This requires PyQt4 to be installed.

Options:
    -m --measurement MPOINT     Specify a measurement point to use (up to 3)
                                can be specified.
    -t --time TIME              Seconds delay between measurements for
                                continuous and debug [default: 0.1]
    -v LEVEL                    Verbosity level: 0, 1 or 2 [default: 0]
"""
from docopt import docopt
import pyenergy
from time import sleep
import usb.util
import textwrap, string
import json, os, os.path
import platformrun
import readline

import logging

def prettyPrint(v):
    units = ['', 'm', 'u', 'n', 'p']

    for unit in units:
        if v > 1.0:
            return "{: >8.3f} {}".format(v, unit)
        v *= 1000.
    return "{}".format(v)

def display(m):
    print "Energy:          {}J".format(prettyPrint(m.energy))
    print "Time:            {}s".format(prettyPrint(m.time))
    print "Power:           {}W".format(prettyPrint(m.avg_power))
    print "Average current: {}A".format(prettyPrint(m.avg_current))
    print "Average voltage: {}V".format(prettyPrint(m.avg_voltage))


def read(serial, pin, mpoints):
    em = pyenergy.EnergyMonitor(serial)
    em.connect()

    for mp in mpoints:
        em.enableMeasurementPoint(mp)
        em.setTrigger(pin, mp)
        em.clearNumberOfRuns(mp)

    for mp in mpoints:
        while not em.measurementCompleted(mp):
            sleep(0.1)

    for mp in mpoints:
        m = em.getMeasurement(mp)
        print "Measurement point",mp
        display(m)
        print ""

def continuous(serial, mpoints, delay=0.1):
    em = pyenergy.EnergyMonitor(serial)
    em.connect()

    print "time,",

    for mp in mpoints:
        em.enableMeasurementPoint(mp)
        em.start(mp)
        print "energy_{0}, power_{0}, avg_current_{0}, avg_voltage_{0},".format(mp),
    print ""

    try:
        while True:
            first = True
            for mp in mpoints:
                m = em.getMeasurement(mp)

                if first:
                    print m.time,
                print "{}, {}, {}, {},".format(m.energy, m.energy/m.time, m.avg_current, m.avg_voltage),
            print ""
            sleep(delay)
    except KeyboardInterrupt:
        for mp in mpoints:
            em.stop(mp)
            em.disableMeasurementPoint(mp)


def debug(serial, mpoints, delay=0.1):
    em = pyenergy.EnergyMonitor(serial)
    em.connect()

    for mp in mpoints:
        em.enableMeasurementPoint(mp)
        em.start(mp)

    try:
        while True:
            for mp in mpoints:
                meas = em.getInstantaneous(mp)
                print "Measurement point:", mp
                em.debugInstantaneous(meas)
            print ""
            sleep(delay)
    except KeyboardInterrupt:
        for mp in mpoints:
            em.stop(mp)
            em.disableMeasurementPoint(mp)


def list_boards():
    devs = pyenergy.EnergyMonitor.getBoards()

    print "Connected energy monitors:"

    if len(devs) > 0:
        print "    Serial  API"
    else:
        print "    None :("

    for d in devs:
        try:
            d.set_configuration()
            v = pyenergy.EnergyMonitor.getVersion(d)

            if v < pyenergy.EnergyMonitor.baseVersion or v < 13:
                serial = "{}(?)".format(usb.util.get_string(d, 3).encode("utf8","ignore"))
            else:
                serial = pyenergy.EnergyMonitor.getSerial(d)
        except usb.core.USBError:
            serial = "Error"
            v = "Error"

        if v < 13:
            v = "{} (Must update firmware before use!)".format(v)
        if v < pyenergy.EnergyMonitor.newestVersion:
            v = "{} (Please update firmware)".format(v)

        print "    {: <8} {}".format(serial, v)

def change_serial(oldser, newser):
    em = pyenergy.EnergyMonitor(oldser)
    em.setSerial(newser)


def preinput(text):
    def f():
        #stdout.write(blue)
        readline.insert_text(text)
        readline.redisplay()
    return f

def get_input(prompt, initial=""):
    readline.set_pre_input_hook(preinput(initial))
    inp = raw_input(prompt)
    # stdout.write(reset)
    return inp

def choice(prompt, choices, initial=""):
    sel = None

    while True:
        sel = get_input("{} [{}] ".format(prompt, "/".join(choices)), initial=initial)
        initial = ""

        if sel == '':
            sel = None
            for c in choices:
                if c.isupper():
                    return c.lower()
        elif sel.lower() in map(string.lower, choices):
            return sel.lower()

def setup():
    list_boards()
    print ""

    text=textwrap.dedent("""
        This program will prompt you for the connections between the measurement
        boards.Necessary information includes the serial of the energy monitor
        connected to which platform, and which pin is used to trigger the energy
        data collection.

        This information is stored in the ~/.measurementrc file. The existing
        config will now be loaded (if there is one), allowing you edit it.
        """)
    print text

    cfg = os.path.expanduser("~/.measurementrc")
    if os.path.exists(cfg):
        config = json.load(open(cfg))
    else:
        print "Configuration file does not exist, creating new config"
        config = {}


    while True:
        c = choice("[A]dd board, [e]dit/[delete] existing, [f]inished?", "aedf")

        if c == "f":
            break
        if c == 'a' or c == "e":
            if c == "a":
                all_platforms = set(platformrun.detect.default_config['platforms'].keys())
                used_platforms = set(config.keys())

                pchoices = list(all_platforms - used_platforms)
            else:
                pchoices = list(config.keys())

            pstrchoices = map(lambda (i, s): "\t[{}] {}".format(i, s), enumerate(pchoices))
            cchoices = "".join(map(str, range(len(pchoices))))

            c1 = choice("Select the platform:\n" + "\n".join(pstrchoices) + "\n ? ", cchoices)
            c1 = int(c1)

            platform = pchoices[c1]

            if c == "e":
                default_ser = config[platform]['energy-monitor']
                default_mp = str(config[platform]['measurement-point'])
                default_res = str(config[platform]['resistor'])
                default_pin = str(config[platform]['trigger-pin'])
            else:
                default_ser = ""
                default_mp = ""
                default_res = ""
                default_pin = ""

            ser = ""
            while ser == "":
                ser = get_input("Serial of energy monitor connected to {}: ".format(platform), initial=default_ser)

            if platform == "beaglebone":
                print "All measurement points needed for beagle bone, autoselecting following config:"
                print "    Measurement points: 1 (DDR), 2 (MPU), 3 (CORE)"
                print "    Resistors         : 0.5,     0.05,    0.05"
                mp = [1,2,3]
                res = [0.5, 0.05, 0.05]
            else:
                mp  = choice("Measurement point connected to {}".format(platform), ["1", "2", "3" ], initial=default_mp)
                res = choice("Shunt resistor used: ", ["0.05", "0.5", "1", "5"], initial = default_res)

                mp = int(mp)
                res = float(res)

            pin = ""
            while pin == "":
                pin = get_input("Set the trigger pin: ", initial=default_pin)

            config[platform] = {"energy-monitor": str(ser), "measurement-point": mp, "resistor": res, "trigger-pin": pin}

            if platform == "atmega328p":
                sdi = get_input("ATMEGA328P requires the id of the serial-USB adapter. This can be\nfound in /dev/serial/by-id/\n ? ")
                config[platform]['serial-dev-id'] = str(sdi)

            print ""
        if c == 'd':
            pchoices = list(config.keys())

            pstrchoices = map(lambda (i, s): "\t[{}] {}".format(i, s), enumerate(pchoices))
            cchoices = "".join(map(str, range(len(pchoices))))

            c1 = choice("Select the platform:\n" + "\n".join(pstrchoices) + "\n ? ", cchoices)
            c1 = int(c1)

            platform = pchoices[c1]
            del config[platform]


    f = open(cfg, "w")
    json.dump(config, f)
    f.close()


def main():
    arguments = docopt(__doc__)

    logging.basicConfig()
    if arguments['-v'] == "1":
        logging.getLogger('').setLevel(logging.INFO)
    if arguments['-v'] == "2":
        logging.getLogger('').setLevel(logging.DEBUG)
        logging.debug("test")

    mpoints = map(int, set(arguments['--measurement']))

    if arguments['read']:
        read(arguments['SERIAL'], arguments['PIN'], mpoints)
    elif arguments['debug']:
        debug(arguments['SERIAL'], mpoints, float(arguments['--time']))
    elif arguments['continuous']:
        continuous(arguments['SERIAL'], mpoints, float(arguments['--time']))
    elif arguments['list']:
        list_boards()
    elif arguments['setup']:
        setup()
    elif arguments['changeserial']:
        change_serial(arguments['SERIAL'], arguments['NEWSERIAL'])
    elif arguments['interactive']:
        try:
            import PyQt4
        except ImportError:
            print "Error: please install PyQt4"
            quit()
        import interactive_graph
        interactive_graph.main(arguments['SERIAL'])

if __name__=="__main__":
    main()



import sys, os, random
from PyQt4.QtCore import *
from PyQt4.QtGui import *

import matplotlib
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar
from matplotlib.figure import Figure
from mpl_toolkits.axes_grid.parasite_axes import SubplotHost

import collections
import numpy as np
import scipy
import scipy.stats

import pyenergy

class Graph(QMainWindow):
    def __init__(self, em, parent=None):
        QMainWindow.__init__(self, parent)
        self.setWindowTitle('Energy monitoring graph')

        self.em = em

        self.tinterval = 0.01
        self.ctime = 0
        self.wsize = 500
        self.toffset = 0
        self.tref = None

        self.create_main()
        self.setUpdate()
        self.setup_graphs()

        self.on_draw()

    def single_graph(self):
        self.fig.clear()
        self.fig.subplots_adjust(right=0.8,left=0.1)

        self.axes = SubplotHost(self.fig, 111)
        self.fig.add_axes(self.axes)
        self.vaxes = self.axes.twinx()

        self.paxes = self.axes.twinx()
        self.paxes.axis["right"].set_visible(False)
        offset = (70, 0)
        new_axisline = self.paxes.get_grid_helper().new_fixed_axis

        self.paxes.axis["side"] = new_axisline(loc="right", axes=self.paxes, offset=offset)
        self.paxes.axis["side"].label.set_visible(True)
        self.paxes.axis["side"].set_label("Power (W)")

        self.axes.set_xlabel("Time (s)")
        self.axes.set_ylabel("Current (A)")
        self.vaxes.set_ylabel("Voltage (V)")
        self.paxes.set_ylabel("Power (W)")

    def horiz_graph(self):
        self.fig.clear()
        self.fig.subplots_adjust(right=0.97,left=0.1, wspace=0.33)

        self.axes = self.fig.add_subplot(131)
        self.vaxes = self.fig.add_subplot(132)
        self.paxes = self.fig.add_subplot(133)

        self.axes.set_xlabel("Time (s)")
        self.vaxes.set_xlabel("Time (s)")
        self.paxes.set_xlabel("Time (s)")

        self.axes.set_ylabel("Current (A)")
        self.vaxes.set_ylabel("Voltage (V)")
        self.paxes.set_ylabel("Power (W)")

    def vert_graph(self):
        self.fig.clear()
        self.fig.subplots_adjust(right=0.97,left=0.1, hspace=0.25)

        self.axes = self.fig.add_subplot(311)
        self.vaxes = self.fig.add_subplot(312)
        self.paxes = self.fig.add_subplot(313)

        self.axes.xaxis.set_ticklabels([])
        self.vaxes.xaxis.set_ticklabels([])
        # self.axes.set_xlabel("Time (s)")
        # self.vaxes.set_xlabel("Time (s)")
        self.paxes.set_xlabel("Time (s)")

        self.axes.set_ylabel("Current (A)")
        self.vaxes.set_ylabel("Voltage (V)")
        self.paxes.set_ylabel("Power (W)")


    def create_main(self):
        self.main_frame = QWidget()

        # Create the mpl Figure and FigCanvas objects.
        # 5x4 inches, 100 dots-per-inch
        #
        self.dpi = 100
        self.fig = Figure((5.0, 4.0), dpi=self.dpi)
        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)
        self.canvas.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

        self.single_graph()

        hbox = QHBoxLayout()
        hbox.setAlignment(Qt.AlignCenter)

        # Create general settings
        grid = QGridLayout()
        grid.setSpacing(5)
        grid.setAlignment(Qt.AlignCenter)

        # Add time slider
        grid.addWidget(QLabel("Interval (s)"), 1, 0)
        self.timeslider = QSlider(1)
        self.timeslider.setMinimum(1)
        self.timeslider.setMaximum(2000)
        self.timeslider.setValue(self.tinterval*1000)
        self.timeslider.setTickInterval(1)
        self.timeslider.setSingleStep(1)
        self.connect(self.timeslider, SIGNAL('valueChanged(int)'), self.updatesliders)
        grid.addWidget(self.timeslider, 2, 0)

        # Add window slider
        grid.addWidget(QLabel("Window size"), 3, 0)
        self.windowslider = QSlider(1)
        self.windowslider.setMinimum(20)
        self.windowslider.setMaximum(2000)
        self.windowslider.setValue(self.wsize)
        self.windowslider.setTickInterval(1)
        self.windowslider.setSingleStep(1)
        self.connect(self.windowslider, SIGNAL('valueChanged(int)'), self.updatesliders)
        grid.addWidget(self.windowslider, 4, 0)

        # Add graph selector
        grid.addWidget(QLabel("Graph stack"), 5, 0)
        self.graphselect = QComboBox()
        self.graphselect.addItems(["Combined", "Horizontal", "Vertical"])
        self.graphselect.setCurrentIndex(0)
        self.connect(self.graphselect, SIGNAL('currentIndexChanged(int)'), self.changegraph)
        grid.addWidget(self.graphselect, 6,0)

        box = QGroupBox("General")
        box.setLayout(grid)
        box.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)

        hbox.addWidget(box)

        self.controls = collections.defaultdict(list)


        for mp in ["1", "2", "3", "Self"]:
            grid = QGridLayout()
            grid.setSpacing(5)
            grid.setAlignment(Qt.AlignCenter)

            combo = QComboBox()
            combo.addItems(["0.05", "0.5", "1", "5"])
            if mp == "Self":
                combo.setCurrentIndex(1)
                combo.setEnabled(False)
            else:
                combo.setCurrentIndex(2)
            self.controls[mp].append(combo)
            grid.addWidget(QLabel("Resistor"), 1, 0)
            grid.addWidget(combo, 1, 1, alignment=Qt.AlignCenter)

            cb = QCheckBox()
            self.controls[mp].append(cb)
            grid.addWidget(QLabel("Plot Current"), 2, 0)
            grid.addWidget(cb, 2, 1, alignment=Qt.AlignCenter)

            cb = QCheckBox()
            self.controls[mp].append(cb)
            grid.addWidget(QLabel("Plot Voltage"), 3, 0)
            grid.addWidget(cb, 3, 1, alignment=Qt.AlignCenter)

            cb = QCheckBox()
            self.controls[mp].append(cb)
            grid.addWidget(QLabel("Plot Power"), 4, 0)
            grid.addWidget(cb, 4, 1, alignment=Qt.AlignCenter)

            l = QLineEdit()
            l.setMinimumWidth(10)
            l.setMaximumWidth(50)
            self.controls[mp].append(l)
            grid.addWidget(QLabel("Label"), 5, 0)
            grid.addWidget(l, 5, 1, alignment=Qt.AlignCenter)

            box = QGroupBox(mp)
            box.setLayout(grid)
            box.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)

            hbox.addWidget(box)

        vbox = QVBoxLayout()
        vbox.addWidget(self.canvas)
        # vbox.addWidget(self.mpl_toolbar)
        vbox.addLayout(hbox)

        self.main_frame.setLayout(vbox)
        self.setCentralWidget(self.main_frame)

    def updatesliders(self):
        self.tinterval = self.timeslider.value() / 1000.
        self.wsize = self.windowslider.value()
        print self.tinterval, self.wsize
        self.setUpdate()
        print self.tinterval

    def changegraph(self):
        ind = self.graphselect.currentIndex()

        self.plots = []

        if ind == 0:
            self.single_graph()
        elif ind == 1:
            self.horiz_graph()
        else:
            self.vert_graph()

    def on_draw(self):
        mintimes = []
        maxtimes = []

        maxps = [0]
        minps = [10]
        maxis = [0]
        minis = [10]
        maxvs = [0]
        minvs = [10]

        for p in self.plots:
            p.remove()
        self.plots = []

        n_points = [0,0,0]

        for s in self.state.values():
            n_points[0] += s[1]
            n_points[1] += s[2]
            n_points[2] += s[3]


        linestyles = [["-","-","-","-"], ["-","-","-","-"], ["-","-","-","-"]]
        # for i in range(3):
        #     if n_points[i] > 1:
        #         linestyles[i] = ["-*", "-+", "-s", "-"]

        for i,(mp, vals) in enumerate(sorted(self.data.items())):
            # Calculate the number of samples in the window
            n = int(len(filter(lambda x: x >= vals["xdata"][-1] - self.tinterval*self.wsize, vals['xdata']))*1.1)

            if self.state[mp][1]:
                p1,  = self.axes.plot(vals["xdata"], vals["idata"], linestyles[0][i], color='g')
                self.plots.append(p1)
                maxis.append(max(vals["idata"][-n:]))
                minis.append(min(vals["idata"][-n:]))

            if self.state[mp][2]:
                p1,  = self.vaxes.plot(vals["xdata"], vals["vdata"], linestyles[1][i], color='b')
                self.plots.append(p1)
                maxvs.append(max(vals["vdata"][-n:]))
                minvs.append(min(vals["vdata"][-n:]))
            if self.state[mp][3]:
                p1,  = self.paxes.plot(vals["xdata"], vals["pdata"], linestyles[2][i], color='r')
                self.plots.append(p1)
                maxps.append(max(vals["pdata"][-n:]))
                minps.append(min(vals["pdata"][-n:]))

            if self.state[mp][1] or self.state[mp][2] or self.state[mp][3]:
                maxtimes.append(max(vals["xdata"]))
                mintimes.append(min(vals["xdata"]))

        mmtimes = self.tinterval * self.wsize
        if len(maxtimes) != 0 and max(maxtimes) > mmtimes:
            mmtimes = max(maxtimes)

        self.axes.set_xlim([ mmtimes - self.tinterval*self.wsize,  mmtimes])
        self.vaxes.set_xlim([ mmtimes - self.tinterval*self.wsize,  mmtimes])
        self.paxes.set_xlim([ mmtimes - self.tinterval*self.wsize,  mmtimes])

        toff = self.tinterval*self.wsize * 0.1

        poff = (max(maxps) - min(minps)) *0.1
        self.paxes.set_ylim([min(minps)-poff, max(maxps)+poff])

        ioff = (max(maxis) - min(minis)) *0.1
        self.axes.set_ylim([min(minis) - ioff, max(maxis) + ioff])

        voff = (max(maxvs) - min(minvs)) *0.1
        self.vaxes.set_ylim([min(minvs)-voff, max(maxvs)+voff])

        bbox_props = dict(boxstyle="round", fc="w", ec="0.5", alpha=0.8)
        for i,(mp, vals) in enumerate(sorted(self.data.items())):
            if self.state[mp][1]:
                l = self.axes.text(vals["xdata"][-1] - toff/2, np.mean(vals["idata"][-self.wsize/10:]), self.state[mp][4], ha="right", bbox=bbox_props)
                self.plots.append(l)
            if self.state[mp][2]:
                l = self.vaxes.text(vals["xdata"][-1] - toff/2, np.mean(vals["vdata"][-self.wsize/10:]), self.state[mp][4], ha="right", bbox=bbox_props)
                self.plots.append(l)
            if self.state[mp][3]:
                l = self.paxes.text(vals["xdata"][-1] - toff/2, np.mean(vals["pdata"][-self.wsize/10:]), self.state[mp][4], ha="right", bbox=bbox_props)
                self.plots.append(l)

        self.canvas.draw()

    def setup_graphs(self):
        self.data = {
            "1": {"xdata": [],
                  "idata": [],
                  "pdata": [],
                  "vdata": []},
            "2": {"xdata": [],
                  "idata": [],
                  "pdata": [],
                  "vdata": []},
            "3": {"xdata": [],
                  "idata": [],
                  "pdata": [],
                  "vdata": []},
            "Self": {"xdata": [],
                  "idata": [],
                  "pdata": [],
                  "vdata": []},
            }
        self.state = {
            "1": [2, False, False, False, "1"],
            "2": [2, False, False, False, "2"],
            "3": [2, False, False, False, "3"],
            "Self": [2, False, False, False, "Self"],
            }
        self.plots = []

    def setUpdate(self):
        self.timer = QTimer()
        self.timer.setSingleShot(False)
        self.timer.timeout.connect(self.update)
        self.timer.start(self.tinterval * 1000)

    def getState(self):
        state = collections.defaultdict(list)
        for mp, vals in self.controls.items():
            for i, control in enumerate(vals):
                if i == 0:
                    state[mp].append(control.currentIndex())
                elif i == 4:
                    state[mp].append(control.text())
                else:
                    state[mp].append(control.checkState() == 2)
        return state


    def update(self):
        state = self.getState()

        self.ctime += self.tinterval

        disabled_tref = None
        measurements = {}
        first_on = None

        for i, mp in enumerate(sorted(state.keys())):
            self.em.measurement_params[i+1]['resistor'] = [0.05, 0.5, 1.0, 5.0][state[mp][0]]
            stateChange = False

            # Check we need to enable or disable a measurement point
            if bool(sum(state[mp][1:4])) != bool(sum(self.state[mp][1:4])):
                stateChange = True
                if bool(sum(state[mp][1:4])):
                    self.em.enableMeasurementPoint(i+1)
                    self.em.start(i+1)

                    self.data[mp]['xdata'] = []
                    self.data[mp]['idata'] = []
                    self.data[mp]['vdata'] = []
                    self.data[mp]['pdata'] = []
                else:
                    self.em.stop(i+1)
                    self.em.disableMeasurementPoint(i+1)
                    if mp == self.tref:
                        disabled_tref = mp

            if bool(sum(state[mp][1:4])) or stateChange:
                m = self.em.getInstantaneous(i+1)
                measurements[mp] = m

                if first_on is None and mp != disabled_tref:
                    first_on = mp

        if self.tref is None or first_on is None:
            self.tref = first_on

        if disabled_tref is not None and first_on is not None:
            self.toffset += (measurements[self.tref][4] - measurements[first_on][4]) * 2. / 168000000*2.
            self.tref = first_on

        if self.tref is not None:
            base_t = measurements[self.tref][4]* 2. / 168000000*2. + self.toffset
        else:
            base_t = 0

        problem = False
        for mp, m in measurements.items():
                i = {"1": 1, "2":2, "3":3, "Self":4}[mp]
                res = self.em.measurement_params[i]['resistor']
                vref = self.em.measurement_params[i]['vref']
                gain = self.em.measurement_params[i]['gain']

                v = float(vref) / 4096. * m[2] * 2
                c = float(vref) / gain / res / 4096. * m[3]
                p = v * c

                t = base_t

                if self.data[mp]['xdata'] and t < self.data[mp]['xdata'][-1]:
                    problem=True

                self.data[mp]['xdata'].append(t)
                self.data[mp]['idata'].append(c)
                self.data[mp]['vdata'].append(v)
                self.data[mp]['pdata'].append(p)

                self.data[mp]['xdata'] = self.data[mp]['xdata'][-self.wsize:]
                self.data[mp]['idata'] = self.data[mp]['idata'][-self.wsize:]
                self.data[mp]['vdata'] = self.data[mp]['vdata'][-self.wsize:]
                self.data[mp]['pdata'] = self.data[mp]['pdata'][-self.wsize:]

        if problem:
            self.data = {
                "1": {"xdata": [],
                      "idata": [],
                      "pdata": [],
                      "vdata": []},
                "2": {"xdata": [],
                      "idata": [],
                      "pdata": [],
                      "vdata": []},
                "3": {"xdata": [],
                      "idata": [],
                      "pdata": [],
                      "vdata": []},
                "Self": {"xdata": [],
                      "idata": [],
                      "pdata": [],
                      "vdata": []},
                }
            

        self.state = state
        self.on_draw()

def main(serial):
    app = QApplication(sys.argv)
    app.setStyle("plastique")

    em = pyenergy.EnergyMonitor(serial)
    em.connect()

    form = Graph(em)
    form.show()
    app.exec_()


if __name__ == "__main__":
    main("EE00")



import usb
import usb.core
import usb.util

from struct import *
from copy import copy
from collections import namedtuple
import operator
from functools import total_ordering

import logging

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())
warning = logger.warning
error = logger.error
info = logger.info
debug = logger.debug


# import multiprocessing
@total_ordering
class MeasurementSet(object):
    def __init__(self, measurements):
        self.measurements = measurements

        self.energy       = sum(self.measurements).energy
        self.time         = sum(self.measurements).time / len(self.measurements)
        self.peak_power   = sum(self.measurements).peak_power  # This is only a rough guide
        self.peak_voltage = sum(self.measurements).peak_voltage
        self.peak_current = sum(self.measurements).peak_current
        self.n_samples    = sum(self.measurements).n_samples
        self.avg_voltage  = sum(self.measurements).avg_voltage
        self.avg_current  = sum(self.measurements).avg_current
        self.avg_power    = sum(self.measurements).avg_power

    def _op_self(self, other, fn):
        params = ["energy", "time", "peak_power", "peak_voltage", "peak_current",
                "n_samples", "avg_voltage", "avg_current", "avg_power"]

        newmeasurements = []
        for m1, m2 in zip(self.measurements, other.measurements):
            m = fn(m1, m2)
            newmeasurements.append(m)
        return MeasurementSet(newmeasurements)

    def _op_const(self, const, fn):
        params = ["energy", "time", "peak_power", "peak_voltage", "peak_current",
                "n_samples", "avg_voltage", "avg_current", "avg_power"]

        newmeasurements = []
        for m1 in self.measurements:
            m = fn(m1, const)
            newmeasurements.append(m)
        return MeasurementSet(newmeasurements)

    def _op(self, other, fn):
        if isinstance(other, MeasurementSet):
            assert len(self.measurements) == len(other.measurements)
            return self._op_self(other, fn)
        else:
            return self._op_const(other, fn);

    def __add__(self, other):
        return self._op(other, operator.add)
    def __sub__(self, other):
        return self._op(other, operator.sub)
    def __mul__(self, other):
        return self._op(other, operator.mul)
    def __div__(self, other):
        return self._op(other, operator.div)

    def __radd__(self, other):
        return self._op(other, operator.add)
    def __rsub__(self, other):
        return self._op(other, operator.sub)
    def __rmul__(self, other):
        return self._op(other, operator.mul)
    def __rdiv__(self, other):
        return self._op(other, operator.div)

    def __repr__(self):
        keys = ["energy", "time", "peak_power", "peak_voltage", "peak_current", "n_samples", "avg_voltage", "avg_current", "avg_power"]
        vals = ", ".join(map(lambda x: "{}={}".format(x,self.__dict__[x]), keys))
        return "MeasurementSet({} measurements, {})".format(len(self.measurements), vals)

    def __eq__(self, rhs):
        return all(map(operator.eq, self.measurements, rhs.measurements))

    def __lt__(self, rhs):
        return self.measurements < rhs.measurements

@total_ordering
class Measurement(object):
    class_version = 1

    def __init__(self, energy=0, time=0, peak_power=0, peak_voltage=0, peak_current=0,
                n_samples=0, avg_voltage=0, avg_current=0, avg_power=0):
        self.energy       = float(energy)
        self.time         = float(time)
        self.peak_power   = float(peak_power)
        self.peak_voltage = float(peak_voltage)
        self.peak_current = float(peak_current)
        self.n_samples    = float(n_samples)
        self.avg_voltage  = float(avg_voltage)
        self.avg_current  = float(avg_current)
        self.avg_power    = float(avg_power)
        self.version      = Measurement.class_version

    def _op_self(self, other, fn):
        params = ["energy", "time", "peak_power", "peak_voltage", "peak_current",
                "n_samples", "avg_voltage", "avg_current", "avg_power"]
        m = Measurement()
        for p in params:
            m.__dict__[p] = fn(self.__dict__[p], other.__dict__[p])
        return m

    def _op_const(self, const, fn):
        params = ["energy", "time", "peak_power", "peak_voltage", "peak_current",
                "n_samples", "avg_voltage", "avg_current", "avg_power"]
        m = Measurement()
        for p in params:
            m.__dict__[p] = fn(self.__dict__[p], const)
        return m

    def _op(self, other, fn):
        if isinstance(other, Measurement):
            return self._op_self(other, fn)
        else:
            return self._op_const(other, fn);

    def __add__(self, other):
        return self._op(other, operator.add)
    def __sub__(self, other):
        return self._op(other, operator.sub)
    def __mul__(self, other):
        return self._op(other, operator.mul)
    def __div__(self, other):
        return self._op(other, operator.div)

    def __radd__(self, other):
        return self._op(other, operator.add)
    def __rsub__(self, other):
        return self._op(other, operator.sub)
    def __rmul__(self, other):
        return self._op(other, operator.mul)
    def __rdiv__(self, other):
        return self._op(other, operator.div)

    def __repr__(self):
        return "Measurement(" + ", ".join(map("{0[0]}={0[1]}".format, self.__dict__.items())) + ")"

    def __eq__(self, rhs):
        params = ["energy", "time", "peak_power", "peak_voltage", "peak_current",
                 "n_samples", "avg_voltage", "avg_current", "avg_power"]

        for p in params:
            if self.__dict__[p] != rhs.__dict__[p]:
                return False

        return True

    def __lt__(self, rhs):
        params = ["energy", "time", "peak_power", "peak_voltage", "peak_current",
                 "n_samples", "avg_voltage", "avg_current", "avg_power"]
        s1 = (self.__dict__[p] for p in params)
        s2 = (rhs.__dict__[p] for p in params)

        return s1 < s2


    # Future proof for new version
    def __getstate__(self):
        return self.__dict__
    def __setstate__(self, d):

        if 'version' not in d or d['version'] < 1:
            # likely to only happen when we have unversioned data
            # so lets make this an info message instead of a warning
            # as we've probably warned about it already
            info("Upgrading to version 1")
            d['version'] = 1
            if "avg_power" not in d:
                d['avg_power'] = d['energy'] / d['time']

        # In future, if this class gets upgraded, add code here to upgrade from
        # previous versions

        if d['version'] > self.class_version:
            error("Measurement was created with a newer version of pyenergy, please upgrade")

        self.__dict__.update(d)

class EnergyMonitor(object):
    """
        The class implements the interface over USB to the MAGEEC enenergy
        monitor boards. This is fairly simple with pyusb, as all of the
        commands are implemented over the control endpoint.

        The class should be instantiated with a serial number, which tells
        it which board to connect to.

            em = EnergyMonitor("EE00")

        Before measuring, the measurement point needs to be enabled. This
        maps an ADC to the measurement point correctly.

            em.enableMeasurementPoint(1)

        After this, either a trigger can be set, or the measurement
        manually start and stopped.

            em.setTrigger("PA0", 1)

            em.start(1)
            em.stop(1)

        The measurement can then be retrieved.

            em.getMeasurement(1)
    """

    MeasurementData = namedtuple('MeasurementData', 'energy_accum elapsed_time peak_power peak_voltage peak_current n_samples avg_current avg_voltage')
    MeasurementData_packing = "=QQLLLLQQ"

    InstantaneousData = namedtuple('InstantaneousData', 'voltage current average_voltage average_current current_time')
    InstantaneousData_packing = "=LLLLQ"

    ADC1 = 0
    ADC2 = 1
    ADC3 = 2

    # These port mappings are the pins connected to voltage and current
    # (respectively) on the stm32f4discovery board.
    port_mappings = {1: ["PA2", "PC2"], 2: ["PA3", "PA1"], 3: ["PB1", "PC5"], 4:["PB0", "PC4"]}

    # The boards did not use to have a version associated with them, the first
    # version is version 10 (thus if we have an error getting the version we
    # assume the board is older than version 10).
    newestVersion = 13
    baseVersion = 10

    def __init__(self, serial="EE00"):
        devs = self.getBoards()

        sdevs = []
        for d in devs:
            try:
                d.set_configuration()
            except usb.core.USBError, e:
                warning("Could not access one of the boards")
                warning(str(e))
            v = self.getVersion(d)
            if v < EnergyMonitor.baseVersion:
                warning("Device attached with old firmware, cannot check if this is desired device")
                continue

            if v < 13:
                if usb.util.get_string(d, 3).encode("utf8","ignore") == serial:
                    error("You need to update the firmware on this board before you can use it")
                continue
            s = self.getSerial(d)

            if s == serial:
                sdevs.append(d)

            # TODO else release config

        devs = sdevs

        if len(devs) > 1:
            warning("More than one device ({}) available with serial {}".format(len(devs), serial))
        if len(devs) == 0:
            raise RuntimeError("Cannot find energy monitor with serial " + serial)

        self.serial = serial
        self.dev = devs[0]

        # Set up default parameters for each measurement point
        defaultparams = {'resistor':1, 'gain':50, 'vref':3}
        self.measurement_params = {i : copy(defaultparams) for i in [1,2,3]}

        # Measurement point 4 is the 'self' measurement point
        self.measurement_params[4] = {'resistor':0.5, 'gain':50, 'vref':3}

        # Equal to tperiod in the firmware
        self.samplePeriod = 500

        self.adcMpoint = [None, None, None]

    @staticmethod
    def getBoards():
        """Return a list of all energy monitor boards connected to the host"""

        devs = list(usb.core.find(idVendor=0xf539, idProduct=0xf539, find_all = True))
        return devs

    # Connect to the device
    def connect(self):
        """Connect to the specified board, and check firmware version"""
        info("Connecting to energy monitor")
        self.version = self.getVersion(self)

        if self.version < EnergyMonitor.baseVersion or self.version < 13:
            error("Firmware is too old, please update")
        if self.version < EnergyMonitor.newestVersion:
            warning("More recent firmware is available, please update")

    # Get version
    @staticmethod
    def getVersion(dev):
        """
            Get the API version of the energy monitoring boards. This may fail
            if the firmware is old and does not implement this request,
            therefore we catch this error.
        """
        try:
            dev = dev.dev
        except:
            pass

        try:
            b = dev.ctrl_transfer(0xc3, 12, 0, 0, 4)
        except usb.core.USBError as e:
            try:
                b = dev.ctrl_transfer(0xc1, 12, 0, 0, 4)
            except usb.core.USBError as e:
                if e.errno == 32:
                    return 0
                raise
        version = unpack("=L", b)[0]
        return version

    # Get serial
    @staticmethod
    def getSerial(dev):
        """Get the serial from the board."""
        try:
            dev = dev.dev
        except:
            pass

        b = dev.ctrl_transfer(0xc3, 13, 0, 0, 4)
        serial = unpack("=4s", b)[0]
        return serial

    # Toggle the LEDs on the device
    def toggleLEDs(self):
        """Toggle some of the LEDs on the board."""
        self.dev.ctrl_transfer(0x43, 0, 0, 0, None)

    # Start measuring on m_point
    def start(self, m_point=1):
        """Clear the number of runs and start measurement on the specified
        measurement point."""
        self.clearNumberOfRuns()
        self.dev.ctrl_transfer(0x43, 1, int(m_point), 0, None)

    # Stop measuring on m_point
    def stop(self, m_point=1):
        """Stop the measurement on the specified measurement point."""
        self.dev.ctrl_transfer(0x43, 2, int(m_point), 0, None)

    # Return whether the measurement point is currently taking
    # measurements or not
    def isRunning(self, m_point=1):
        """Check whether a particular measurement point is running."""
        debug("Check if running")
        b = self.dev.ctrl_transfer(0xc3, 8, int(m_point), 0, 4)
        debug("Received bytes: {0}".format(b))

        running = unpack("=L", b)
        debug("Running: {0}".format(running))
        return bool(running[0])

    # This counts the number of end measurement signals caught
    # by the energy monitor.
    def getNumberOfRuns(self, m_point=1):
        """
            Return the number of 'runs' seen by the energy monitor. This is
            the number of times a full measurement has been taken since the
            last clearNumberOfRuns call.

            This is mostly used as part of checking whether a triggered
            measurement has completed, in measurementComplete.
        """

        debug("Get number of runs")
        b = self.dev.ctrl_transfer(0xc3, 9, int(m_point), 0, 4)

        runs = unpack("=L", b)
        debug("Runs: {0}".format(runs[0]))
        return int(runs[0])

    # Reset the number of runs counts to 0
    def clearNumberOfRuns(self, m_point=1):
        """Clear the counter used to store the number of runs."""
        self.dev.ctrl_transfer(0x43, 10, int(m_point), 0, None)

    # Have we completed a measurement?
    def measurementCompleted(self, m_point = 1):
        """
            Check whether a full measurement has been taken. Also clear the
            number of runs if we are waiting for a trigger.
        """

        runs = self.getNumberOfRuns(m_point)
        if runs > 1:
            warning("More than one measurement has completed (expected one)")
        if not self.isRunning(m_point) and runs > 0:
            self.clearNumberOfRuns(m_point)
            return True
        return False

    # Set the serial number
    def setSerial(self, ser):
        """Set the board's serial number. Must be exactly 4 characters long."""

        if len(ser) != 4:
            warning("Serial should be 4 characters.")
            ser = (ser + "0000")[:4]
        self.dev.ctrl_transfer(0x43, 3, ord(ser[0]) | (ord(ser[1])<<8), ord(ser[2]) | (ord(ser[3])<<8), None)

    # Set a particular port as a pin trigger for a measurement point
    #   e.g PA0
    def setTrigger(self, port, m_point=1):
        """
            Set up a trigger on the specified port and measurement point. The
            board will automatically start measuring when the pin rises, and
            stop when the pin falls.

            The port should be of the form P[A-H][0-9]. Not all pins on the
            board will work, due to peripheral multiplexing and additional
            components attached to some pins.

            TODO: a list of acceptable and tested pins (PA0 works).
        """

        # TODO check port is of the form PA0
        info("Set trigger on {}".format(port))
        self.dev.ctrl_transfer(0x43, 4, ord(port[1]) | (m_point<<8), int(port[2]), None)

    # Enable a particular measurement point. There are
    # only 3 ADCs in the device, so only 3 measurement points
    # can be used simultaneously
    def enableMeasurementPoint(self, m_point, adc=None):
        """
            Enable a measurement point on the board.

            There are only 3 ADCs and 4 measurement points. Additionally
            certain ADCs can only measure certain measurement points,
            hence this function to map an ADC to a measurement point.

            This function records which measurement points are in use,
            the attempts to map an ADC to it, if any are free.
        """

        if m_point in self.adcMpoint:
            warning("Tried to enable already enabled measurement point "+str(m_point))
            return
        if self.adcMpoint.count(None) == 0:
            raise RuntimeError("Cannot enable measurement point {}. Maximum of enabled measurement points reached".format(m_point))
        if adc is not None and self.adcMpoint[adc] is not None:
            raise RuntimeError("Cannot enable map measurement point {0} to ADC{1}. ADC{1} already has measurement point {2}".format(m_point, adc, self.adcMpoint[adc]))

        if adc is None:
            # If we want mpoint 1 or 2 and ADC3 is free, prioritise it
            #  becuase ADC3 doesnt work with mpoint 3 or self
            if m_point in [1, 2] and self.adcMpoint[2] is None:
                adc = 2
            else:
                adc = self.adcMpoint.index(None)
        if m_point in [3,4] and adc == 2:
            raise RuntimeError("Measurement point {} cannot be used with ADC3 (the only free ADC)".format(m_point))
        info("Measurement point {} mapped to ADC {}".format(m_point, adc))
        self.adcMpoint[adc] = m_point
        self.dev.ctrl_transfer(0x43, 7, int(m_point), adc, None)

    # Disable a particular measurement point, so that the
    # ADC could potentially be used with a different one
    def disableMeasurementPoint(self, m_point):
        """
            To disable the measurement point we simply remove it from the set
            of mapped measurement points.

            We also send a stop command, to ensure no further measurements are
            taken.
        """

        info("Disable measurement point {}".format(m_point))

        if m_point not in self.adcMpoint:
            warning("Tried to disable already disabled measurement point "+str(m_point))
            return
        adc = self.adcMpoint.index(m_point)
        self.adcMpoint[adc] = None

        # TODO remove the trigger
        self.stop()

    def convertData(self, md, resistor, gain, vref, samplePeriod):
        """
            Convert the data from the raw form returned, and into standard
            units.
        """
        try:
            en = float(vref)**2 / gain / resistor / 4096**2 * 2 * samplePeriod * 2 / 168000000. * md.energy_accum * 2
            el = md.elapsed_time * 2. / 168000000 * 2
            pp = float(vref)**2 / gain / resistor / 4096**2 * md.peak_power * 2
            pv = float(vref) / 4096. * md.peak_voltage * 2
            pi = float(vref) / gain / resistor / 4096. * md.peak_current
            av = float(vref) / 4096. * md.avg_voltage / md.n_samples * 2
            ai = float(vref) / gain / resistor / 4096. * md.avg_current / md.n_samples
            ap = en/el
        except ZeroDivisionError:
            en, el, pp, pv, pi, av, ai, ap = 0, 0, 0, 0, 0, 0, 0, 0

        m = Measurement(en, el, pp, pv, pi, md.n_samples, av, ai, ap)

        return m

    def getMeasurement(self, m_point=1):
        """
            Get the most recent measurement taken for a particular measurement
            point.
        """

        info("getMeasurement")
        b = self.dev.ctrl_transfer(0xc3, 6, int(m_point), 0, calcsize(EnergyMonitor.MeasurementData_packing))
        u = EnergyMonitor.MeasurementData._make(unpack(EnergyMonitor.MeasurementData_packing, b))

        info(u)

        return self.convertData(u, samplePeriod=self.samplePeriod, **self.measurement_params[m_point])

    # get an instantaneous measurement of voltage and current (debugging)
    def getInstantaneous(self, m_point=1):
        """
            This is mostly a debug function that returns the instantaneous
            current and voltage, as well as an average of the current and
            voltage over the last 32 samples.
        """
        b = self.dev.ctrl_transfer(0xc3, 11, int(m_point), 0, calcsize(EnergyMonitor.InstantaneousData_packing))
        args = list(unpack(EnergyMonitor.InstantaneousData_packing, b))
        args.append(m_point)
        return args

    # Convert and display instantaneous measurement
    def debugInstantaneous(self, v):
        """
            Display the data returned in getInstantaneous.
        """
        mp = v[5]
        resistor = self.measurement_params[mp]['resistor']
        gain = self.measurement_params[mp]['gain']
        vref = self.measurement_params[mp]['vref']

        print "Timestamp:", v[4] * 2. / 168000000 * 2
        print "Current:  Raw={:4d}  Voltage@{}={:1.3f}V  Res Vdrop={:1.5f}V  Current={:1.5f}A".format(mp,
            EnergyMonitor.port_mappings[mp][1],
            v[3]/4096.*vref,
            float(vref) / gain / 4096. * v[3],
            float(vref) / gain / resistor / 4096. * v[3])
        print "Voltage:  Raw={:4d}  Voltage@{}={:1.3f}V                      Voltage={:1.5f}V".format(v[2],
            EnergyMonitor.port_mappings[mp][0],
            v[2]/4096.*vref,
            float(vref) / 4096. * v[2] * 2)
        print ""

    def disconnect(self):
        """For now this doesn't do anything"""
        pass



if __name__ == "__main__":
    from time import sleep

    em = EnergyMonitor("EE00")
    em.connect()

    em.toggleLEDs()

    em.enableMeasurementPoint(1)
    em.enableMeasurementPoint(2)

    em.setTrigger("PA0", 1)
    em.setTrigger("PA0", 2)

    print "*** Press the blue button to make a measurement"

    while True:
        while not em.measurementCompleted(): sleep(0.1)
        print em.getMeasurement(1)
        print em.getMeasurement(2)



#!/usr/bin/python

"""Energy measurement upgrade tool.

This tools loads pickled objects, upgrades any Measurement types it finds
and then writes the objects back to the file. This is mostly useful for
data saved in the old namedtuple Measurement format, as other measurement
will get upgraded on load anyway.

Usage:
    measurement-upgrade [-v | -vv] FILENAME

Options
    --verbose -v        Be verbose
"""

import pyenergy
import logging
from docopt import docopt

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())
warning = logger.warning
error = logger.error
info = logger.info


### Start ugly, ugly hack here ################################################
# This hack is here to maintain backwards compatability for Measurements
# already pickled, using the old namedtuple approach. The new Measurement
# class has at least the old functionality of the old tuple (except it is not)
# a tuple.
#
# This hack modifies the pickle module to use a different method of
# instantiating a class when it discovers an old namedtuple class. It then uses
# the arguments to create a new style Measurement class
import pickle, sys

def _reconstructor_hack(cls, base, state):
    if base is object:
        obj = object.__new__(cls)
    else:
        # Detect whether the cls and base are old style, and if so
        # create the object using its constructor, instead of tuple's __new__
        if cls == pyenergy.Measurement and base is tuple:
            if not hasattr(_reconstructor_hack, "warned"):
                warning("Loading from unversioned pickled file.")
                _reconstructor_hack.warned = True
            return pyenergy.Measurement(*state)
        obj = base.__new__(cls, state)
        if base.__init__ != object.__init__:
            base.__init__(obj, state)
    return obj

def find_class_hack(self, module, name):
    if name == "_reconstructor":
        return _reconstructor_hack

    __import__(module)
    mod = sys.modules[module]
    klass = getattr(mod, name)
    return klass

### End ugly hack #############################################################

def updateOldPickle(fname):

    oldfind = pickle.Unpickler.find_class
    pickle.Unpickler.find_class = find_class_hack

    objs = []
    f = open(fname, "r")
    while True:
        try:
            o = pickle.load(f)
            objs.append(o)
        except EOFError:
            break
    f.close()

    print o[0][0]

    f = open(fname, "w")
    for o in objs:
        pickle.dump(o, f)

    f.close()

    pickle.Unpickler.oldfind = find_class_hack

def main():
    arguments = docopt(__doc__)

    logging.basicConfig()

    if arguments['--verbose'] == 1:
        logging.getLogger('').setLevel(logging.INFO)
    elif arguments['--verbose']== 2:
        logging.getLogger('').setLevel(logging.DEBUG)

    updateOldPickle(arguments['FILENAME'])

if __name__ == "__main__":
    main()



"""
Usage:
    platformrun-detect [options]

Options:
    -c --config FILE    Specify the tools configuration file
                        [default: ~/.platformrunrc]
    -p --prompt         Prompt for locations of tools not found
    --clean             Overwrite config with a clean configuration

This tool will help set up the configuration for platform run.
This will try and detect the prerequisites needed to run on
each platform and enter them into a configuration file.

"""
from docopt import docopt
import json, os, os.path, sys
import copy
import readline, glob

default_config = {
    'tools': {
        'arm_gdb' : 'arm-none-eabi-gdb',
        'stutil' : 'st-util',
        'openocd' : 'openocd',
        'avr_objcopy' : "avr-objcopy",
        'avrdude' : "avrdude",
        'pic32_objcopy' : 'pic32-objcopy',
        'pic32prog' : 'pic32prog',
        'mspdebug': 'mspdebug',
        'openocd': 'openocd',
    },
    'platforms': {
        'stm32f0discovery': ['arm_gdb', 'stutil'],
        'stm32vldiscovery': ['arm_gdb', 'stutil'],
        'stm32f4discovery': ['arm_gdb', 'stutil'],
        'stm32l0discovery': ['arm_gdb', 'openocd'],
        'beaglebone': ['arm_gdb', 'openocd'],
        'atmega328p': ['avr_objcopy', 'avrdude'],
        'xmegaa3buxplained': ['avr_objcopy', 'avrdude'],
        'pic32mx250f128b': ['pic32_objcopy', 'pic32prog'],
        'msp-exp430f5529' : ['mspdebug'],
        'msp-exp430fr5739' : ['mspdebug'],
        'sam4lxplained' : ['arm_gdb','openocd'],
    },
    'enabled': []
}

# This is directly taken from shutils.which (not yet backported to python 2.X)
def which(cmd, mode=os.F_OK | os.X_OK, path=None):
    """Given a command, mode, and a PATH string, return the path which
    conforms to the given mode on the PATH, or None if there is no such
    file.

    `mode` defaults to os.F_OK | os.X_OK. `path` defaults to the result
    of os.environ.get("PATH"), or can be overridden with a custom search
    path.

    """
    # Check that a given file can be accessed with the correct mode.
    # Additionally check that `file` is not a directory, as on Windows
    # directories pass the os.access check.
    def _access_check(fn, mode):
        return (os.path.exists(fn) and os.access(fn, mode)
                and not os.path.isdir(fn))

    # If we're given a path with a directory part, look it up directly rather
    # than referring to PATH directories. This includes checking relative to the
    # current directory, e.g. ./script
    if os.path.dirname(cmd):
        if _access_check(cmd, mode):
            return cmd
        return None

    if path is None:
        path = os.environ.get("PATH", os.defpath)
    if not path:
        return None
    path = path.split(os.pathsep)

    if sys.platform == "win32":
        # The current directory takes precedence on Windows.
        if not os.curdir in path:
            path.insert(0, os.curdir)

        # PATHEXT is necessary to check on Windows.
        pathext = os.environ.get("PATHEXT", "").split(os.pathsep)
        # See if the given file matches any of the expected path extensions.
        # This will allow us to short circuit when given "python.exe".
        # If it does match, only test that one, otherwise we have to try
        # others.
        if any(cmd.lower().endswith(ext.lower()) for ext in pathext):
            files = [cmd]
        else:
            files = [cmd + ext for ext in pathext]
    else:
        # On other platforms you don't have things like PATHEXT to tell you
        # what file suffixes are executable, so just pass on cmd as-is.
        files = [cmd]

    seen = set()
    for dir in path:
        normdir = os.path.normcase(dir)
        if not normdir in seen:
            seen.add(normdir)
            for thefile in files:
                name = os.path.join(dir, thefile)
                if _access_check(name, mode):
                    return name

def detect_prog(progname, prompt=False):

    def preinput(text):
        def f():
            readline.insert_text(text)
            readline.redisplay()
        readline.set_pre_input_hook(f)

    if not os.path.exists(progname):
        f = which(progname)
    else:
        f = progname

    if f is None and prompt:
        path = None

        preinput(progname)

        while True:
            path = raw_input("Enter location of \"{}\"\n > ".format(progname))

            if path == "":
                f = None
                break

            path = os.path.expanduser(path)

            if os.path.isdir(path):
                if os.path.exists(os.path.join(path, progname)):
                    f = os.path.join(path, progname)
                    break
                else:
                    print '"{}" cannot be found under directory "{}"'.format(progname,path)
            else:
                if os.path.exists(path):
                    f = path
                    break
                else:
                    print '"{}" cannot be found'.format(path)


    return f

def main():
    arguments = docopt(__doc__)

    # Set up file completion
    def complete(text, state):
        return (glob.glob(os.path.expanduser(text)+'*')+[None])[state]

    readline.set_completer_delims(' \t\n;')
    readline.parse_and_bind("tab: complete")
    readline.set_completer(complete)

    # Load existing config
    cfg = os.path.expanduser(arguments['--config'])
    if os.path.exists(cfg):
        config = json.load(open(cfg))
    else:
        print "Configuration file does not exist, creating new config"
        config = copy.deepcopy(default_config)

    # Try and detect if the programs exist
    for tool, name in default_config['tools'].items():
        try:
            if not isinstance(config['tools'][tool], str) and not isinstance(config['tools'][tool], unicode):
                config['tools'][tool] = default_config['tools'][tool]
        except KeyError:
            config['tools'][tool] = default_config['tools'][tool]
        config['tools'][tool] = detect_prog(config['tools'][tool], prompt=arguments['--prompt'])

    # For each platform, check we have the dependencies
    config['enabled'] = []
    for platform in default_config['platforms'].keys():
        haveTools = True
        for tool in default_config['platforms'][platform]:
            if config['tools'][tool] is None:
                haveTools = False

        if platform not in config['platforms']:
            config['platforms'][platform] = default_config['platforms'][platform]

        if haveTools:
            config['enabled'].append(platform)

    # Write out the config file
    json.dump(config, open(os.path.expanduser(arguments['--config']), "w+"),
        indent=2)

    # Display a summary
    print "\nSummary\n"
    for platform in default_config['platforms'].keys():
        print "{: <20}      {}".format(platform, "enabled" if platform in config['enabled'] else "disabled")

if __name__=="__main__":
    main()
